#include "widget.hpp"
#include "gfx.hpp"
#include <glm/gtc/matrix_transform.hpp>
#include <GLFW/glfw3.h>

Widget::Widget(Orbit& orbit, GLuint shaderProgram) :
    _orbit(orbit),
    _shaderProgram(shaderProgram)
{
    const float lineLength = 0.1;
    for (int i = 0; i < 3*6; i++)
    {
        _vertices[i] *= _lineLength;
    }

    glGenVertexArrays(1, &_vao);
    glGenBuffers(1, &_vbo);

    glBindVertexArray(_vao);
    glBindBuffer(GL_ARRAY_BUFFER, _vbo);
    size_t v3Size = 3 * sizeof(float);
    size_t lineSize = 2 * sizeof(float);
    size_t vboBufferSize = v3Size * lineSize * 3;
    glBufferData(GL_ARRAY_BUFFER, vboBufferSize, &_vertices[0], GL_STATIC_DRAW);

    glEnableVertexAttribArray(0);
    glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 3 * sizeof(float), (void*)0);

    glBindBuffer(GL_ARRAY_BUFFER, 0);
    glBindVertexArray(0);
}

void Widget::render()
{
    updateModelMatrix();

    glBindVertexArray(_vao);
    glBindBuffer(GL_ARRAY_BUFFER, _vbo);
    glDrawArrays(GL_LINES, 0, 6);
}

void Widget::updateModelMatrix()
{
    float p = glfwGetTime();

    _position = _orbit.getPosition(p);

    // get the tangent of the orbital ellipse
    glm::vec3 tan = _orbit.getTangent(p);
    // we want to point along the orbit
    glm::vec3 target = _position + tan;
    // 'up' is just the normalized position vector because the orbit is centred at the origin
    glm::vec3 up = glm::normalize(_position);
    // easy peasy lookAt matrix
    glm::mat4 look = glm::lookAt(_position, target, up);

    // invert the lookat matrix because it's meant for cameras, cameras work backwards and
    // we are not a camera
    glm::mat4 model = glm::inverse(look);

    GLint modelLocation = getShaderUniformLocation(_shaderProgram, "_Model");
    glUniformMatrix4fv(modelLocation, 1, GL_FALSE, &model[0][0]);
}

Widget::~Widget()
{
    glDeleteVertexArrays(1, &_vao);
    glDeleteBuffers(1, &_vbo);
}
